# fshof

FsHof (**F**ile**s**ystem **H**igher-**o**rder **f**unctions) provides a useful
functional interface for filesystem interactions.

Here is how you could get the total number of lines of all txt files in the
current directory:

```python
import fshof as fs

fs.Dir(".").files.filter_ext("txt").map_lc().sum()
```

The library is very flexible, allowing you to write both short and long forms for
most properties:

```python
import fshof as fs

# Short form (for when you quickly need to prototype something)
fs.Dir(".").files.filter_ext("txt").map_lc().sum()

# Long form (for when you need to use the library in a codebase)
fs.Dir(".").files.filter_extension("txt").map_line_count().sum()
```

You can also pass your own functions to the higher-order functions:

```python
import fshof as fs

# Short form (for when you quickly need to prototype something)
fs.Dir(".").files.filter(lambda f: f.ext == "txt").map(lambda f: f.t().lc).sum()

# Long form (for when you need to use the library in a codebase)
fs.Dir(".").files.filter(lambda f: f.extension == "txt").map(lambda f: f.text_file().line_count).sum()
```

## Requirements

You need Python >= 3.9 for this library.

## Installation
