from enum import Enum
from typing import Any


class FileSizeUnit(Enum):
    BYTE = 0
    KB = 1
    MB = 2
    GB = 3
    TB = 4
    KIB = 5
    MIB = 6
    GIB = 7
    TIB = 8

    def __str__(self) -> str:
        return {
            FileSizeUnit.BYTE: "B",
            FileSizeUnit.KB: "KB",
            FileSizeUnit.MB: "MB",
            FileSizeUnit.GB: "GB",
            FileSizeUnit.TB: "TB",
            FileSizeUnit.KIB: "KiB",
            FileSizeUnit.MIB: "MiB",
            FileSizeUnit.GIB: "GiB",
            FileSizeUnit.TIB: "TiB",
        }[self]


class FileSize:
    ConversionValues = {
        FileSizeUnit.BYTE: 1,
        FileSizeUnit.KB: 1e3,
        FileSizeUnit.MB: 1e6,
        FileSizeUnit.GB: 1e9,
        FileSizeUnit.TB: 1e12,
        FileSizeUnit.KIB: 1024,
        FileSizeUnit.MIB: 1024**2,
        FileSizeUnit.GIB: 1024**3,
        FileSizeUnit.TIB: 1024**4,
    }

    def __init__(self, size_bytes: int) -> None:
        self.size_bytes = size_bytes

    def __int__(self) -> int:
        return self.size_bytes

    def __add__(self, other: Any) -> "FileSize":
        return FileSize(int(self) + int(other))

    def __radd__(self, other: Any) -> "int":
        return self + other

    def size(self, unit: FileSizeUnit = FileSizeUnit.BYTE) -> float:
        return self.size_bytes / FileSize.ConversionValues[unit]

    def size_readable(
        self, unit: FileSizeUnit = FileSizeUnit.BYTE, rounding: int = 2
    ) -> str:
        return f"{round(self.size(unit), rounding)}{str(unit)}"

    def __eq__(self, other: Any) -> bool:
        return self.size_bytes == other.size_bytes

    def __str__(self) -> str:
        return repr(self)

    def __repr__(self) -> str:
        return self.size_readable()
