from collections.abc import Iterator
from functools import reduce
from typing import TypeVar, Callable, List, Iterable, Any

T = TypeVar("T")


class FunctionalIterator(Iterator[T]):
    def __init__(self, iterable: Iterable[T]) -> None:
        super().__init__()
        self.it = iter(iterable)

    def __next__(self) -> T:
        return next(self.it)

    def map(self, fun: Callable[[T], Any]) -> "FunctionalIterator[Any]":
        return FunctionalIterator(map(fun, self))

    def filter(self, fun: Callable[[T], bool]) -> "FunctionalIterator[T]":
        return FunctionalIterator(filter(fun, self))

    def reduce(self, fun: Callable[[Any, T], T], start: Any) -> Any:
        return reduce(fun, self, start)

    def sum(self) -> int:
        return self.reduce(lambda acc, val: acc + val, 0)

    def len(self) -> int:
        return self.reduce(lambda acc, val: acc + 1, 0)

    def list(self) -> List[T]:
        return list(self)
